@@ -2,7 +2,7 @@
  *
  * Copyright (C) 2003-2004  Narcis Ilisei <inarcis2002@hotpop.com>
  * Copyright (C) 2006       Steve Horbachuk
- * Copyright (C) 2010-2017  Joachim Nilsson <troglobit@gmail.com>
+ * Copyright (C) 2010-2020  Joachim Nilsson <troglobit@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,18 +21,19 @@
  * Boston, MA  02110-1301, USA.
  */
 
-#include <arpa/nameser.h>
-#include <net/if.h>
-#include <netinet/in.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/ioctl.h>
-#include <sys/time.h>
-#include <sys/types.h>
 #include <ifaddrs.h>
 #include <time.h>
 #include <unistd.h>
+#include <sys/ioctl.h>
+#include <sys/socket.h>
+#include <sys/time.h>
+#include <sys/types.h>
+#include <netinet/in.h>
+#include <arpa/nameser.h>
+#include <net/if.h>
 
 #include "ddns.h"
 #include "cache.h"
@@ -155,7 +156,8 @@ static int server_transaction(ddns_t *ctx, ddns_info_t *provider)
 		rc = RC_DDNS_INVALID_CHECKIP_RSP;
 
 	http_exit(client);
-	logit(LOG_DEBUG, "Checked my IP, return code: %d", rc);
+	logit(LOG_DEBUG, "Server response: %s", trans->rsp);
+	logit(LOG_DEBUG, "Checked my IP, return code %d: %s", rc, error_str(rc));
 
 	return rc;
 }
@@ -541,6 +543,9 @@ static int send_update(ddns_t *ctx, ddns_info_t *info, ddns_alias_t *alias, int
 	http_trans_t   trans;
 	http_t        *client = &info->server;
 
+	if (info->system->setup)
+		DO(info->system->setup(ctx, info, alias));
+
 	client->ssl_enabled = info->ssl_enabled;
 	rc = http_init(client, "Sending IP# update to DDNS server");
 	if (rc) {
@@ -574,13 +579,14 @@ static int send_update(ddns_t *ctx, ddns_info_t *info, ddns_alias_t *alias, int
 	logit(LOG_DEBUG, "Sending alias table update to DDNS server: %s", ctx->request_buf);
 
 	rc = http_transaction(client, &trans);
-	logit(LOG_DEBUG, "DDNS server response: %s", trans.rsp);
-
 	if (rc) {
 		/* Update failed, force update again in ctx->cmd_check_period seconds */
+		logit(LOG_WARNING, "HTTP(S) Transaction failed, error %d: %s", rc, error_str(rc));
+		logit(LOG_INFO, "Update failed, forced update/retry in %d sec ...", ctx->cmd_check_period);
 		ctx->force_addr_update = 1;
 		goto exit;
 	}
+	logit(LOG_DEBUG, "DDNS server response: %s", trans.rsp);
 
 	rc = info->system->response(&trans, info, alias);
 	if (rc) {
@@ -665,7 +671,7 @@ static int update_alias_table(ddns_t *ctx)
 				os_shell_execute(script_exec, alias->address, alias->name);
 		}
 
-		if (RC_DDNS_RSP_NOTOK == rc)
+		if (RC_DDNS_RSP_NOTOK == rc || RC_DDNS_RSP_AUTH_FAIL == rc)
 			remember = rc;
 
 		if (RC_DDNS_RSP_RETRY_LATER == rc && !remember)
@@ -700,9 +706,11 @@ static int get_encoded_user_passwd(void)
 
 		info->creds.encoded = 0;
 
-		/* Concatenate username and password with a ':', without
+		/*
+		 * Concatenate username and password with a ':', without
 		 * snprintf(), since that can cause information loss if
-		 * the password has "\=" or similar in it, issue #57 */
+		 * the password has "\=" or similar in it, issue #57
+		 */
 		strlcpy(buf, info->creds.username, len);
 		strlcat(buf, ":", len);
 		strlcat(buf, info->creds.password, len);
@@ -717,7 +725,7 @@ static int get_encoded_user_passwd(void)
 			break;
 		}
 
-		logit(LOG_DEBUG, "Base64 encode %s for %s ...", buf, info->system->name);
+//		logit(LOG_DEBUG, "Base64 encode %s for %s ...", buf, info->system->name);
 		rc2 = base64_encode((unsigned char *)encode, &dlen, (unsigned char *)buf, strlen(buf));
 		if (rc2) {
 			logit(LOG_WARNING, "Failed base64 encoding user:pass for %s!", info->system->name);
@@ -734,6 +742,7 @@ static int get_encoded_user_passwd(void)
 		info = conf_info_iterator(0);
 	}
 
+	memset(buf, 0, len);
 	free(buf);
 
 	return rc;
@@ -829,6 +838,7 @@ static int check_error(ddns_t *ctx, int rc)
 		break;
 
 	case RC_DDNS_RSP_NOTOK:
+	case RC_DDNS_RSP_AUTH_FAIL:
 		if (ignore_errors) {
 			logit(LOG_WARNING, "%s, ignoring ...", errstr);
 			break;
@@ -888,11 +898,11 @@ int ddns_main_loop(ddns_t *ctx)
 	DO(read_cache_file(ctx));
 	DO(get_encoded_user_passwd());
 
-	if (once == 1)
+	if (once && force)
 		ctx->force_addr_update = 1;
 
 	/* Initialization done, create pidfile to indicate we are ready to communicate */
-	if (once == 0 && pidfile(pidfile_name))
+	if (once == 0 && pidfile_name[0] && pidfile(pidfile_name))
 		logit(LOG_WARNING, "Failed creating pidfile: %s", strerror(errno));
 
 	/* DDNS client main loop */
