@@ -1,6 +1,6 @@
 /* GnuTLS interface for optional HTTPS functions
  *
- * Copyright (C) 2014-2017  Joachim Nilsson <troglobit@gmail.com>
+ * Copyright (C) 2014-2020  Joachim Nilsson <troglobit@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,6 +19,7 @@
  * Boston, MA 02110-1301, USA.
  */
 
+#include <stdint.h>
 #include <gnutls/x509.h>
 
 #include "log.h"
@@ -62,8 +63,11 @@ static int verify_certificate_callback(gnutls_session_t session)
 	if (status & GNUTLS_CERT_EXPIRED)
 		logit(LOG_WARNING, "The certificate has expired.");
 
-	if (status & GNUTLS_CERT_NOT_ACTIVATED)
+	if (status & GNUTLS_CERT_NOT_ACTIVATED) {
 		logit(LOG_WARNING, "The certificate is not yet activated.");
+		if (broken_rtc && (status &= ~GNUTLS_CERT_NOT_ACTIVATED) == GNUTLS_CERT_INVALID)
+			status = 0;
+	}
 
 	if (status & GNUTLS_CERT_INVALID) {
 		logit(LOG_ERR, "The certificate is not trusted.");
@@ -284,7 +288,7 @@ int ssl_send(http_t *client, const char *buf, int len)
 	if (ret < 0)
 		return RC_HTTPS_SEND_ERROR;
 
-	logit(LOG_DEBUG, "Successfully sent DDNS update using HTTPS!");
+	logit(LOG_DEBUG, "Successfully sent HTTPS request!");
 
 	return 0;
 }
@@ -301,8 +305,11 @@ int ssl_recv(http_t *client, char *buf, int buf_len, int *recv_len)
 		ret = gnutls_record_recv(client->ssl, buf, len);
 	} while (ret == GNUTLS_E_INTERRUPTED || ret == GNUTLS_E_AGAIN);
 
-	if (ret < 0)
+	if (ret < 0) {
+		logit(LOG_WARNING, "Failed receiving GnuTLS header response: %s",
+		      gnutls_strerror(ret));
 		return RC_HTTPS_RECV_ERROR;
+	}
 
 	/* Read HTTP body */
 	len = ret;
@@ -315,11 +322,14 @@ int ssl_recv(http_t *client, char *buf, int buf_len, int *recv_len)
 		}
 	} while (ret == GNUTLS_E_INTERRUPTED || ret == GNUTLS_E_AGAIN);
 
-	if (ret < 0)
+	if (ret < 0 && ret != GNUTLS_E_PREMATURE_TERMINATION) {
+		logit(LOG_WARNING, "Failed receiving GnuTLS body response: %s",
+		      gnutls_strerror(ret));
 		return RC_HTTPS_RECV_ERROR;
+	}
 
 	*recv_len = len;
-	logit(LOG_DEBUG, "Successfully received DDNS update response (%d bytes) using HTTPS!", len);
+	logit(LOG_DEBUG, "Successfully received HTTPS response (%d bytes)!", len);
 
 	return 0;
 }
