@@ -1,6 +1,6 @@
 /* OpenSSL interface for optional HTTPS functions
  *
- * Copyright (C) 2014-2017  Joachim Nilsson <troglobit@gmail.com>
+ * Copyright (C) 2014-2020  Joachim Nilsson <troglobit@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,17 +25,21 @@
 
 int ssl_init(void)
 {
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
 	SSL_library_init();
 	SSL_load_error_strings();
 	OpenSSL_add_all_algorithms();
-
+#endif
+	
 	return 0;
 }
 
 void ssl_exit(void)
 {
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
 	ERR_free_strings();
 	EVP_cleanup();
+#endif
 }
 
 static int verify_callback(int preverify_ok, X509_STORE_CTX *ctx)
@@ -65,9 +69,12 @@ static int verify_callback(int preverify_ok, X509_STORE_CTX *ctx)
 		X509_STORE_CTX_set_error(ctx, err);
 	}
 
-	if (!preverify_ok)
+	if (!preverify_ok) {
 		logit(LOG_ERR, "Certificate verification error:num=%d:%s:depth=%d:%s",
 		      err, X509_verify_cert_error_string(err), depth, buf);
+		if (broken_rtc && err == X509_V_ERR_CERT_NOT_YET_VALID)
+			preverify_ok = 1;
+	}
 
 	/*
 	 * At this point, err contains the last verification error. We can use
@@ -106,11 +113,31 @@ done:
 	return 0;
 }
 
+static int ssl_error_cb(const char *str, size_t len, void *data)
+{
+	size_t sz;
+	char buf[512];
+
+	memset(buf, 0, sizeof(buf));
+	sz = len < sizeof(buf) ? len : sizeof(buf) - 1;
+	memcpy(buf, str, sz);
+
+	logit(LOG_ERR, "OpenSSL error: %s", buf);
+
+	return 0;
+}
+
+static void ssl_check_error(void)
+{
+	ERR_print_errors_cb(ssl_error_cb, NULL);
+}
+
 int ssl_open(http_t *client, char *msg)
 {
-	char buf[256];
 	const char *sn;
+	char buf[512];
 	X509 *cert;
+	int rc;
 
 	if (!client->ssl_enabled)
 		return tcp_init(&client->tcp, msg);
@@ -124,8 +151,11 @@ int ssl_open(http_t *client, char *msg)
 		return RC_HTTPS_OUT_OF_MEMORY;
 
 	/* POODLE, only allow TLSv1.x or later */
-	SSL_CTX_set_options(client->ssl_ctx, SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3 | SSL_OP_NO_COMPRESSION);
-
+#ifndef OPENSSL_NO_EC
+	SSL_CTX_set_options(client->ssl_ctx, SSL_OP_SINGLE_ECDH_USE | SSL_OP_SINGLE_DH_USE | SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3 | SSL_OP_NO_COMPRESSION);
+#else
+	SSL_CTX_set_options(client->ssl_ctx, SSL_OP_SINGLE_DH_USE | SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3 | SSL_OP_NO_COMPRESSION);
+#endif
 	SSL_CTX_set_verify(client->ssl_ctx, SSL_VERIFY_PEER, verify_callback);
 	SSL_CTX_set_verify_depth(client->ssl_ctx, 150);
 
@@ -143,8 +173,11 @@ int ssl_open(http_t *client, char *msg)
 		return RC_HTTPS_SNI_ERROR;
 
 	SSL_set_fd(client->ssl, client->tcp.socket);
-	if (-1 == SSL_connect(client->ssl))
+	rc = SSL_connect(client->ssl);
+	if (rc < 0) {
+		ssl_check_error();
 		return RC_HTTPS_FAILED_CONNECT;
+	}
 
 	logit(LOG_INFO, "SSL connection using %s", SSL_get_cipher(client->ssl));
 
@@ -168,12 +201,18 @@ int ssl_open(http_t *client, char *msg)
 int ssl_close(http_t *client)
 {
 	if (client->ssl_enabled) {
-		/* SSL/TLS close_notify */
-		SSL_shutdown(client->ssl);
+		if (client->ssl) {
+			/* SSL/TLS close_notify */
+			SSL_shutdown(client->ssl);
 
-		/* Clean up. */
-		SSL_free(client->ssl);
-		SSL_CTX_free(client->ssl_ctx);
+			/* Clean up. */
+			SSL_free(client->ssl);
+			client->ssl = NULL;
+		}
+		if (client->ssl_ctx) {
+			SSL_CTX_free(client->ssl_ctx);
+			client->ssl_ctx = NULL;
+		}
 	}
 
 	return tcp_exit(&client->tcp);
@@ -181,40 +220,57 @@ int ssl_close(http_t *client)
 
 int ssl_send(http_t *client, const char *buf, int len)
 {
-	int err;
+	int rc, err = SSL_ERROR_NONE;
 
 	if (!client->ssl_enabled)
 		return tcp_send(&client->tcp, buf, len);
 
-	err = SSL_write(client->ssl, buf, len);
-	if (err <= 0)
-		/* XXX: TODO add SSL_get_error() to figure out why */
+	ERR_clear_error();
+	if((rc = SSL_write(client->ssl, buf, len)) <= 0){
+		err = SSL_get_error(client->ssl, rc);
+		if(err == SSL_ERROR_WANT_WRITE){
+			return ssl_send(client, buf, len);
+		}
+	}
+	if (rc <= 0) {
+		ssl_check_error();
 		return RC_HTTPS_SEND_ERROR;
+	}
 
-	logit(LOG_DEBUG, "Successfully sent DDNS update using HTTPS!");
+	logit(LOG_DEBUG, "Successfully sent HTTPS request!");
 
 	return 0;
 }
 
 int ssl_recv(http_t *client, char *buf, int buf_len, int *recv_len)
 {
-	int len, err;
+	int len, rc, err = SSL_ERROR_NONE;
 
+	*recv_len = 0;
 	if (!client->ssl_enabled)
 		return tcp_recv(&client->tcp, buf, buf_len, recv_len);
 
+	ERR_clear_error();
 	/* Read HTTP header */
-	len = err = SSL_read(client->ssl, buf, buf_len);
-	if (err <= 0)
-		/* XXX: TODO add SSL_get_error() to figure out why */
+	len = rc = SSL_read(client->ssl, buf, buf_len);
+	if (rc <= 0) {
+		err = SSL_get_error(client->ssl, rc);
+		if(err == SSL_ERROR_WANT_READ){
+			return ssl_recv(client, buf, buf_len, recv_len);
+		}
+		ssl_check_error();
 		return RC_HTTPS_RECV_ERROR;
+	}
+	*recv_len += len;
 
 	/* Read HTTP body */
-	*recv_len = SSL_read(client->ssl, &buf[len], buf_len - len);
-	if (*recv_len <= 0)
-		*recv_len = 0;
+	len = rc = SSL_read(client->ssl, &buf[len], buf_len - len);
+	if (rc <= 0) {
+		ssl_check_error();
+		len = 0;
+	}
 	*recv_len += len;
-	logit(LOG_DEBUG, "Successfully received DDNS update response (%d bytes) using HTTPS!", *recv_len);
+	logit(LOG_DEBUG, "Successfully received HTTPS response (%d bytes)!", *recv_len);
 
 	return 0;
 }
